%******************
% Interpolacion ENO
%******************

%Funcin: int_eno
%******************

%ENTRADA:
% niv - Niveles de zoom.
% a - Imagen a interpolar.
% met_particion - 0 para no jerarquico, 1 para jerarquico..

%SALIDA:
% b - Imagen interpolada.

function [b] = int_eno(niv, a, met_particion)

[n m] = size(a);

% Bulce para los niveles de zoom.
for k=1:niv
    
    % Filas y columnas para la imagen interpolada.
    n_filas = 2*n; n_columnas = 2*m;
    
    % Creamos la nueva matriz.
    b = zeros(n_filas, n_columnas);
    b(1:2:n_filas, 1:2:n_columnas) = a(1:n, 1:m);

    % Algoritmo para la prediccion de las columnas.
    for j=1:n_columnas
        b(1:n_filas,j) = eno_zoom(b(1:2:n_filas, j), n, met_particion)';
    end

    % Algoritmo para la prediccion de las filas.
    for i =1:n_filas
        b(i, 1:n_columnas) = eno_zoom(b(i,1:2:n_columnas), m, met_particion);
        
    end

    
    % Acutalizaciones de las variables.
    n = n_filas;
    m = n_columnas;
    
    % Inicializacion matriz de zoom.
    a = b;
end
b = uint8(b);
end

% Funcin: eno_zoom
% ******************

% ENTRADA:
% v - Vector fila al que se le va a aplicar el algoritmo.
% n - Longitud del vector v.
% met_particion - 0 para jerarquico, 1 para no jerarquico..
% SALIDA:
% f - Vector fila con valores interpolados.

function f = eno_zoom(v, n, met_particion)

% Creamos el vector f, vector de salida de la funcion.
f = zeros(1, 2*n);
f(1:2:2*n-1) = v(1:n);

% Obtencion de la mascaras ENO.
[m1] = getMask(4/2, 4/2);
[m3] = getMask(4/2+1, 4/2-1);
[m2] = getMask(4/2-1, 4/2+1);

% Prediccion del primer elemento con mascara lineal;
f(2) = m2(1)*v(1) + m2(2)*v(2) + m2(3)*v(3) + m2(4)*v(4);

if(met_particion == 0) % metodo no querarquico
    
    % Prediccion del segundo elemento.
    
    % Cculo de las diferencias dividas.
    elem2 = abs(-v(1) + 3*v(2) - 3*v(3) + v(4));
    elem3 = abs(-v(2) + 3*v(3) - 3*v(4) + v(5));
    if(elem2<elem3)
         f(4) = m1(1)*v(1) + m1(2)*v(2) + m1(3)*v(3) + m1(4)*v(4);
    else
        f(4) = m2(1)*v(2) + m2(2)*v(3) + m2(3)*v(4) + m2(4)*v(5);
    end
    % Prediccion valores intermedios.
    for j=6:2:2*n-6
        elem1 = abs(-v(j/2-2) + 3*v(j/2-1) - 3*v(j/2) + v(j/2+1));
        elem2 = abs(-v(j/2-1) + 3*v(j/2) - 3*v(j/2+1) + v(j/2+2));
        elem3 = abs(-v(j/2) + 3*v(j/2+1) - 3*v(j/2+2) + v(j/2+3));
        if(elem2 <= elem1 && elem2 < elem3)
            f(j) = m1(1)*v(j/2-1) + m1(2)*v(j/2) + m1(3)*v(j/2+1)+ m1(4)*v(j/2+2); % Si es s2
        else
            if(elem1 <= elem3)
                f(j) = m3(1)*v(j/2-2) + m3(2)*v(j/2-1) + m3(3)*v(j/2) + m3(4)*v(j/2+1);
            else
                f(j) = m2(1)*v(j/2) + m2(2)*v(j/2+1) + m2(3)*v(j/2+2) + m2(4)*v(j/2+3);
            end
        end
    end
% Prediccion del penulitmo elemento.
    elem1 = abs(-v(n-4) + 3*v(n-3) - 3*v(n-2) + v(n-1));
    elem2 = abs(-v(n-3) + 3*v(n-2) - 3*v(n-1) + v(n));
    if(elem2 <= elem1)
        f(2*n-4) = m1(1)*v(n-3) + m1(2)*v(n-2) + m1(3)*v(n-1) + m1(4)*v(n);
    else
        f(2*n-4) =  m3(1)*v(n-4) + m3(2)*v(n-3) + m3(3)*v(n-2) + m3(4)*v(n-1);
    end
    % Prediccion del ultimo elemento
    f(2*n-2) = m2(1)*v(n) + m2(2)*v(n-1) + m2(3)*v(n-2) + m2(4)*v(n-3);


else %met_particion = 'jerarquico'
    elem1 = (v(2) - v(1));
    elem2 = (v(3) - v(2));
    elem3 = (v(4) - v(3));
    elem4 = (v(5) - v(4));
    dif1 = (elem2 - elem1); a_dif1 = abs(dif1);
    dif2 = (elem3 - elem2); a_dif2 = abs(dif2);

    if(a_dif1 < a_dif2)
        f(4) = m1(1)*v(1) + m1(2)*v(2) + m1(3)*v(3) + m1(4)*v(4);
    else
            dif3 = elem4 - elem3;
            aux1= (dif2 - dif1); a_aux1 = abs(aux1);
            aux2 = (dif3 - dif2); a_aux2 = abs(aux2);
            if(a_aux1 <= a_aux2)
                f(4) = m1(1)*v(1) + m1(2)*v(2) + m1(3)*v(3) + m1(4)*v(4);
            else
                f(4) = m2(1)*v(2) + m2(2)*v(3) + m2(3)*v(4) + m2(4)*v(5);
            end
    end
    for j=6:2:2*n-6
        elem1 = (v(j/2-1) - v(j/2-2));
        elem2 = (v(j/2) - v(j/2-1));
        elem3 = (v(j/2+1) - v(j/2));
        elem4 = (v(j/2+1) - v(j/2+1));
        elem5 = (v(j/2+3) - v(j/2+2));
        dif1 = (elem2 - elem1); 
        dif2 = (elem3 - elem2); a_dif2 = abs(dif2);
        dif3 = (elem4 - elem3); a_dif3 = abs(dif3);
        dif4 = (elem5 - elem4); 
        if(a_dif2 <= a_dif3)
            aux1 = (elem2 - elem1); a_aux1 = abs(aux1);
            aux2 = (elem3 - elem2); a_aux2 = abs(aux2);
            if(a_aux2 <= a_aux1)
                f(j) = m1(1)*v(j/2-1) + m1(2)*v(j/2) + m1(3)*v(j/2+1)+ m1(4)*v(j/2+2);
            else
                f(j) = m3(1)*v(j/2-2) + m3(2)*v(j/2-1) + m3(3)*v(j/2) + m3(4)*v(j/2+1);
            end
        else
            aux2 = (dif3 - dif2); a_aux2 = abs(aux2);
            aux3= (dif4 - dif3);  a_aux3 = abs(aux3);
            if(a_aux2 <= a_aux3)
                f(j) = m1(1)*v(j/2-1) + m1(2)*v(j/2) + m1(3)*v(j/2+1)+ m1(4)*v(j/2+2);
            else
                f(j) = m2(1)*v(j/2) + m2(2)*v(j/2+1) + m2(3)*v(j/2+2) + m2(4)*v(j/2+3);
            end
        end
    end
 % Prediccion penultimo elemento
 elem1 = (v(n-3) - v(n-4));
 elem2 = (v(n-2) - v(n-3));
 elem3 = (v(n-1) - v(n-2));
 elem4 = (v(n) - v(n-1));
 dif1 = (elem2 - elem1); a_dif1 = abs(dif1); 
 dif2 = (elem3 - elem2); a_dif2 = abs(dif2);
 dif3 = (elem4 - elem3); a_dif3 = abs(dif3);
 if(a_dif3 <= a_dif2)
     f(2*n-4) = m1(1)*v(n-3) + m1(2)*v(n-2) + m1(3)*v(n-1) + m1(4)*v(n);
 else
     aux1 = (dif2 - dif1); a_aux1 = abs(aux1);
     aux2 = (dif3 - dif2); a_aux2 = abs(aux2);
     if(a_aux2 <= a_aux1)
         f(2*n-4) = m1(1)*v(n-3) + m1(2)*v(n-2) + m1(3)*v(n-1) + m1(4)*v(n);
     else
         f(2*n-4) = m3(1)*v(n-4) + m3(2)*v(n-3) + m3(3)*v(n-2) + m3(4)*v(n-1);
     end
 end        
end 
% Prediccion del ultimo elemento

f(2*n-2) = m3(1)*v(n-3) + m3(2)*v(n-2) + m3(3)*v(n-1) + m3(4)*v(n);
end