%*********************
% Interpolacion ENO-SR
%*********************

%Funcin: int_enoSR
%******************

%ENTRADA:
% niv - Niveles de zoom.
% a - Imagen a interpolar.


%SALIDA:
% b - Imagen interpolada.

function [ b ] = int_enoSR(niv, a)
[n m] = size(a);

% Bulce para los niveles de zoom.
for k=1:niv
 % Filas y columnas de la imagen interpolada.
    n_filas = 2*n;
    n_columnas = 2*m;
    
    % Creamos la nueva matriz.
    b = zeros(n_filas, n_columnas);
    b(1:2:n_filas, 1:2:n_columnas) = a(1:n, 1:m);
    
    % Algoritmo de prediccion para las columnas.
    for j=1:2:n_columnas
       b(1:n_filas, j) = enoSR_zoom(b(1:2:n_filas, j), n)';
    end
    
    % Algorimo de preciccion para las filas.
    for i =1:n_filas
       b(i, 1:n_columnas) = enoSR_zoom(b(i,1:2:n_columnas), m);
    end
    
    % Actualizacin de variables.
    n = n_filas;
    m = n_columnas;
    
    % Inicializacin de la matriz de zoom.
    a = b;
end

b = uint8(b);
end

% Funcin: enosr_zoom
% ******************

% ENTRADA:
% v - Vector fila al que se le va a aplicar el algoritmo.
% n - Longitud del vector v.

% SALIDA:
% f - Vector fila con valores interpolados.

function [f] = enoSR_zoom(v, n)

% Cramos el vector f, vector de salida de la funcion.
f = zeros(1, 2*n);
f(1:2:2*n-1) = v(1:n);

% Obtencion de la mascaras.
orden=4;
[m1] = getMask(orden/2, orden/2);
[m3] = getMask(orden/2+1, orden/2-1);
[m2] = getMask(orden/2-1, orden/2+1);

% Prediccion del primer elemento con mascara lineal;
f(2) = m2(1)*v(1) + m2(2)*v(2) + m2(3)*v(3) + m2(4)*v(4);

% Prediccion del segundo elemento.
    
% Cculo de las diferencias dividas.
elem1 = (v(2) - v(1));
elem2 = (v(3) - v(2));
elem3 = (v(4) - v(3));
elem4 = (v(5) - v(4));
dif1 = (elem2 - elem1); a_dif1 = abs(dif1);
dif2 = (elem3 - elem2); a_dif2 = abs(dif2);

if(a_dif1 <= a_dif2)
    f(4) = m1(1)*v(1) + m1(2)*v(2) + m1(3)*v(3) + m1(4)*v(4);
else
    dif3 = (elem4 - elem3);
    t1 = dif2 - dif1; a_t1 = abs(t1);
    t2 = dif3 - dif2; a_t2 = abs(t2);
    if(a_t1 <= a_t2)
        f(4) = m1(1)*v(1) + m1(2)*v(2) + m1(3)*v(3) + m1(4)*v(4);
    else
        f(4) = m2(1)*v(2) + m2(2)*v(3) + m2(3)*v(4) + m2(4)*v(5);
    end
end

% Prediccion del tercer elemento con ENO jerarquico.
elem1 = (v(2) - v(1));
elem2 = (v(3) - v(2));
elem3 = (v(4) - v(3));
elem4 = (v(5) - v(4));
elem5 = (v(6) - v(5));
dif1 = (elem2 - elem1); a_dif1 = abs(dif1);
dif2 = (elem3 - elem2); a_dif2 = abs(dif2);
dif3 = (elem4 - elem3); a_dif3 = abs(dif3);
dif4 = (elem5 - elem4); a_dif4 = abs(dif4);
  
if(a_dif2<=a_dif3)
    t1 = a_dif2 - a_dif1; a_t1 = abs(t1);
    t2 = a_dif3 - a_dif2; a_t2 = abs(t2);
    if(a_t2 <= a_t1)
        f(6) = m1(1)*v(2) + m1(2)*v(3) + m1(3)*v(4)+ m1(1)*v(5);
    else
        f(6) = m3(1)*v(1) + m3(2)*v(2) + m3(3)*v(3) + m3(4)*v(4);
    end
else
    t2 = dif2 - dif2; a_t2= abs(t2);
    t3 = dif4 - dif3; a_t3 = abs(t3);
    if(a_t2 <= a_t3)
        f(6) = m1(1)*v(3) + m1(2)*v(4) + m1(3)*v(5)+ m1(1)*v(6);
    else
        f(6) = m3(1)*v(3) + m3(2)*v(4) + m3(3)*v(5) + m(4)*v(6);
    end
end
   
% Prediccion de valores intermedios
for j=8:2:2*n-8
    %Paso 1 : para cada intervalo determinamos los stencil a la izquierda
    %y a la derecha.. Si no esta descentrado aplicamos eno-h.
    s_izq = stencil(v(j/2-3:j/2+2));
    s_der = stencil(v(j/2-1:j/2+4));
    if(s_izq == 'I' & s_der == 'D')
        %Continuamos con el metodode resolucion subcelda comprobamos si
        %g(x_i*g(x_(i+1)) <0.
        q11 = v(j/2);
        q12 = -v(j/2-1) + 4*v(j/2-2) - 6*v(j/2-1) + 4*v(j/2);
        q21 = 3*v(j/2+1) - 6*v(j/2+2)+ 4*v(j/2+3) - v(j/2+4);
        q22 = (j/2+1);
        
        g1 = q21 - q11;
        g2 = q22 - q12;
        
        if(g1*g2 < 0)
            %Hay una discontinuidad en el intervalo.
            q1m = -5/16*v(j/2-3) + 21/16*v(j/2-2) - 35/16*v(j/2-1) + 35/16*v(j/2);
            q2m = 35/16*v(j/2+1) - 35/16*v(j/2+2) - 21/16*v(j/2+3) - 5/16*v(j/2+4);
            
            %funcion g evaluada en punto medio.
            gm = q2m - q1m;
            if(g1*gm < 0)
                f(j) = q2m;
            else
                f(j)= q1m;
            end
        else
            %ENO jerarquico.
            p1 = v(j/2-1) - v(j/2-2);
            p2 = v(j/2) - v(j/2-1);
            p3 = v(j/2+1) - v(j/2);
            p4 = v(j/2+2) - v(j/2+1);
            p5 = v(j/2+3) - v(j/2+2);
            dif1 = p2 - p1;
            dif2 = p3 - p2;
            dif3 = p4 - p3;
            dif4 = p5 - p4;
            
            if(as2 <= as3)
                aux1 = dif2 - dif1; a_aux1 = abs(aux1);
                aux2 = dif3 - dif2; a_aux2 = abs(aux2);
                
                if(a_aux2 <= a_aux1)
                    f(j) =  m1(1)*v(1) + m1(2)*v(2) + m1(3)*v(3) + m1(4)*v(4);
                else
                    f(j) = m3(1)*v(j/2-2) + m3(2)*v(j/2-1) + m3(3)*v(j/2) + m3(4)*v(j/2+1);
                end
            else
                aux2 = dif3 - dif2; a_aux2 = abs(aux2);
                aux3 = dif3 - dif3; a_aux3 = abs(aux3);
                if(a_aux2 <= a_aux3)
                    f(j) =  m1(1)*v(1) + m1(2)*v(2) + m1(3)*v(3) + m1(4)*v(4);
                else
                     f(j) = m2(1)*v(j/2) + m2(2)*v(j/2+1) + m2(3)*v(j/2+2) + m2(4)*v(j/2+3);
                end
            end
        end
    else
        % Aplicamos ENO jerrquico.
        elem1 = (v(j/2-1) - v(j/2-2));
        elem2 = (v(j/2) - v(j/2-1));
        elem3 = (v(j/2+1) - v(j/2));
        elem4 = (v(j/2+1) - v(j/2+1));
        elem5 = (v(j/2+3) - v(j/2+2));
        dif1 = (elem2 - elem1); 
        dif2 = (elem3 - elem2); a_dif2 = abs(dif2);
        dif3 = (elem4 - elem3); a_dif3 = abs(dif3);
        dif4 = (elem5 - elem4);
        
        if(a_dif2 <= a_dif3)
            aux1 = (elem2 - elem1); a_aux1 = abs(aux1);
            aux2 = (elem3 - elem2); a_aux2 = abs(aux2);
            if(a_aux2 <= a_aux1)
                f(j) = m1(1)*v(j/2-1) + m1(2)*v(j/2) + m1(3)*v(j/2+1)+ m1(4)*v(j/2+2);
            else
                f(j) = m3(1)*v(j/2-2) + m3(2)*v(j/2-1) + m3(3)*v(j/2) + m3(4)*v(j/2+1);
            end
        else
            aux2 = (dif3 - dif2); a_aux2 = abs(aux2);
            aux3= (dif4 - dif3); a_aux3 = abs(aux3);
            if(a_aux2 <= aux3)
                f(j) = m1(1)*v(j/2-1) + m1(2)*v(j/2) + m1(3)*v(j/2+1)+ m1(4)*v(j/2+2);
            else
                f(j) = m2(1)*v(j/2) + m2(2)*v(j/2+1) + m2(3)*v(j/2+2) + m2(4)*v(j/2+3);
            end
        end
    end
end

% Prediccion penultimo elemento.
elem1 = (v(n-4) - v(n-5));
elem2 = (v(n-3) - v(n-4));
elem3 = (v(n-2) - v(n-3));
elem4 = (v(n-1) - v(n-2));
elem5 = (v(n) - v(n-1));
dif1 = (elem2 - elem1); a_dif1 = abs(dif1); 
dif2 = (elem3 - elem2); a_dif2 = abs(dif2);
dif3 = (elem4 - elem3); a_dif3 = abs(dif3);
dif4 = (elem5 - elem4);
if(a_dif2 <= a_dif3)
    aux1 = (elem2 - elem1); a_aux1 = abs(aux1);
    aux2 = (elem3 - elem2); a_aux2 = abs(aux2);
    if(a_aux2 <= a_aux1)
        f(2*n-6) = m1(1)*v(n-4) + m1(2)*v(n-3) + m1(3)*v(n-2) + m1(4)*v(n-1);
    else
        f(2*n-6) = m3(1)*v(n-5) + m3(2)*v(n-4) + m3(3)*v(n-3) + m3(4)*v(n-2);
    end
else
    aux2 = (elem3 - elem2); a_aux2 = abs(aux2);
    aux3 = (elem4 - elem3); a_aux3 = abs(aux2);
    if(a_aux2 <= a_aux3)
        f(2*n-6) = m1(1)*v(n-4) + m1(2)*v(n-3) + m1(3)*v(n-2) + m1(4)*v(n-1);
    else
        f(2*n-6) = m2(1)*v(n-3) + m2(2)*v(n-2) + m2(3)*v(n-1) + m2(4)*v(n);
    end    
end

% Prediccion penultimo elemento.
elem1 = (v(n-3) - v(n-4));
elem2 = (v(n-2) - v(n-3));
elem3 = (v(n-1) - v(n-2));
elem4 = (v(n) - v(n-1));
dif1 = (elem2 - elem1); a_dif1 = abs(dif1); 
dif2 = (elem3 - elem2); a_dif2 = abs(dif2);
dif3 = (elem4 - elem3); a_dif3 = abs(dif3);
if(a_dif3 <= a_dif2)
    f(2*n-4) = m1(1)*v(n-3) + m1(2)*v(n-2) + m1(3)*v(n-1) + m1(4)*v(n);
else
    aux1 = (dif2 - dif1); a_aux1 = abs(aux1);
    aux2 = (dif3 - dif2); a_aux2 = abs(aux2);
    if(a_aux2 <= a_aux1)
        f(2*n-4) = m1(1)*v(n-3) + m1(2)*v(n-2) + m1(3)*v(n-1) + m1(4)*v(n);
    else
        f(2*n-4) = m3(1)*v(n-4) + m3(2)*v(n-3) + m3(3)*v(n-2) + m3(4)*v(n-1);
    end
end
% Prediccion del ultimo elemento.
f(2*n-2) = m2(1)*v(n) + m2(2)*v(n-1) + m2(3)*v(n-2) + m2(4)*v(n-3);
end
     
% Funcin: stencil
% ******************

% ENTRADA:
% v - .

% SALIDA:
% r - .

function [r] = stencil(v)
  elem1 = (v(2) - v(1));
  elem2 = (v(3) - v(2));
  elem3 = (v(4) - v(3));
  elem4 = (v(5) - v(4));
  elem5 = (v(6) - v(5));
  dif1 = (elem2 - elem1); a_dif1 = abs(dif1);
  dif2 = (elem3 - elem2); a_dif2 = abs(dif2);
  dif3 = (elem4 - elem3); a_dif3 = abs(dif3);
  dif4 = (elem5 - elem4);
  
  if(a_dif2 <= a_dif3)
      t1 = dif2 - dif1; a_t1 = abs(t1);
      t2 = dif3 - dif2; a_t2 = abs(t2);
      if(a_t2 <= a_t1)
          r = 'C';
      else
          r = 'I';
      end
  else
      t2 = dif3 - dif2; a_t2 = abs(t2);
      t3 = dif4 - dif3; a_t3 = abs(t3);
      if(a_t2 <= a_t3)
          r = 'C';
      else
          r = 'I';
      end
  end
end
