%******************
% Interpolacion PPH
%******************

%Funcin: int_pph
%******************

%ENTRADA:
% niv - Niveles de zoom.
% a - Imagen a interpolar.

%SALIDA:
% b - Imagen interpolada.

function [b]=int_pph(niv, a)

 [f, c] = size(a);

% Bulce para los niveles de zoom.
 for k=1:niv
     
     % Filas y columnas para la imagen interpolada.
     nf = 2*f;
     nc = 2*c;
     
     % Creamos la nueva matriz.
     b = zeros(nf, nc);
     b(1:2:nf, 1:2:nc) = a(1:f, 1:c);
         
     % Algoritmo para la prediccion de las columnas.
     for j=1:2:nc
        b(1:nf, j) = pphzoom2(b(1:2:nf, j), f)';
     end

     %Prediccion de las filas.
     for i=1:nf
        b(i, 1:nc) = pphzoom2(b(i, 1:2:nc), c);
     end
     
     % Acutalizaciones de las variables.
     f = nf;
     c = nc;
     
     % Inicializacion matriz de zoom.
     a = b;
 end
 
  b = uint8(b);
end


%Funcin: int_pph
%******************

% ENTRADA:
% v - Vector fila al que se le va a aplicar el algoritmo.
% n - Longitud del vector v.

% SALIDA:
% f - Vector fila con valores interpolados.

function f = pphzoom2(v, n)

% Creamos el vector f, vector de salida de la funcion.
f = zeros(1, 2*n);
f(1:2:2*n-1) = v(1:n);

% Calculo de las mascaras.
[m2] = getMask(4/2-1, 4/2+1);

% Prediccion del primer elemento.
f(2) = m2(1)*v(1) + m2(2)*v(2) + m2(3)*v(3) + m2(4)*v(4);

% Prediccion de los valores intermedios.
for j=4:2:2*n-4
    elem1 = v(j/2-1)-2*v(j/2)+v(j/2+1);
    elem2 = v(j/2)-2*v(j/2+1)+v(j/2+2);
    prod = elem1*elem2;
    aux = (v(j/2)+v(j/2+1))/2;
    if(prod > 0)
        f(j) = aux-prod/(4*(elem1+elem2));
    else
        f(j) = aux;
    end
end

% Prediccion del ultimo elemento.
f(2*n-2) = m2(1)*v(n) + m2(2)*v(n-1) + m2(3)*v(n-2) + m2(4)*v(n-3);
end

