%******************
% Interpolacion Racional
%******************

%Funcin: int_racional
%******************

%ENTRADA:
% niv - Niveles de zoom.
% a - Imagen a interpolar.

%SALIDA:
% b - Imagen interpolada.

function [ b ] = int_racional(niv, a)

[n m] = size(a);

%Bucle para los niveles de zoom.
for k=1:niv
    
    % Filas y columnas para la imagen interpolada.
    n_filas = 2*n;
    n_columnas = 2*m;
    
    % Creamos la nueva matriz.
    b=zeros(n_filas,n_columnas);
    b(1:2:n_filas,1:2:n_columnas) = a(1:n,1:m);
    
    % Algoritmo de prediccion para las columnas.
    for j=1:2:n_columnas
        b(1:n_filas,j) = racional_zoom(b(1:2:n_filas, j), n)';
    end
    
    % Algorimo de preciccion para las filas.
    for i =1:n_filas
       b(i, 1:n_columnas) = racional_zoom(b(i,1:2:n_columnas),m);
    end
    
    %Actualizacin de variables.
    n = n_filas;
    m = n_columnas;
    
    % Inicializacion matriz de zoom.
    a = b;
end

b = uint8(b);
end

% Funcin: racional_zoom
% ******************

% ENTRADA:
% v - Vector fila al que se le va a aplicar el algoritmo.
% n - Longitud del vector v.

% SALIDA:
% f - Vector fila con valores interpolados.


function [f] = racional_zoom(v, n)

f = zeros(1, 2*n);
f(1:2:2*n-1) = v(1:n);
orden=4;
% Obtencion de la mascara ENO.
[m1] = getMask(orden/2, orden/2);
[m3] = getMask(orden/2-1, orden/2+1);
[m2] = getMask(orden/2+1, orden/2);

% Prediccion del primer elemento.
f(2) = m1(1)*v(1) + m1(2)*v(2) + m1(3)*v(3) + m1(4)*v(4);

% Prediccion del segundo elemento.
elem2 = abs(-v(1) + 3*v(2) - 3*v(3) + v(4));
elem3 = abs(-v(2) + 3*v(3) - 3*v(4) + v(5));
if(elem2 < elem3)
     f(4) = m1(1)*v(1) + m1(2)*v(2) + m1(3)*v(3) + m1(4)*v(4);
else
    f(4) = m2(1)*v(2) + m2(2)*v(3) + m2(3)*v(4) + m2(4)*v(5);
end
% Prediccion valores intermedios
epsilon = 10^(-6);
for j=6:2:2*n-4
     elem1 = (v(j/2+1)-v(j/2+2));
     elem2 = (v(j/2-1) - v(j/2));
     
     %pesos
     a = 1/2;
     w0 = (1 + a*(elem1^2))/(2+a*(elem2^2+elem1^2));
     w1 = (1 + a*(elem2^2))/(2+a*(elem2^2+elem1^2));
     f(j) = w0*v(j/2) + w1*v(j/2+1);
end
 
% Prediccion del ultimo elemento.
f(2*n-2) = m2(1)*v(n) + m2(2)*v(n-1) + m2(3)*v(n-2) + m2(4)*v(n-3);
         
     

end



